@@ -1,7 +1,7 @@
 /* 
  * Seahorse
  * 
- * Copyright (C) 2005 Nate Nielsen 
+ * Copyright (C) 2005 Stefan Walter
  * 
  * This program is free software; you can redistribute it and/or modify 
  * it under the terms of the GNU Lesser General Public License as
@@ -176,16 +176,15 @@ lookup_key_property (CryptUIKeyset *keyset, const gchar *key, const gchar *prop,
     }
     
     value = g_new0 (GValue, 1);
-    if (!dbus_g_proxy_call (keyset->priv->remote_service, "GetKeyField", &error,
+    if (!dbus_g_proxy_call (keyset->priv->remote_keyset, "GetKeyField", &error,
                             G_TYPE_STRING, key, G_TYPE_STRING, prop, G_TYPE_INVALID, 
-                            G_TYPE_VALUE, value, G_TYPE_INVALID)) {
+                            G_TYPE_BOOLEAN, allocated, G_TYPE_VALUE, value, G_TYPE_INVALID)) {
         g_warning ("dbus call to get '%s' failed: %s", prop, error ? error->message : "");
         g_clear_error (&error);
         g_free (value);
         return NULL;
     }
 
-    *allocated = TRUE;
     return value;
 }
 
@@ -375,12 +374,59 @@ cryptui_keyset_class_init (CryptUIKeysetClass *klass)
  * PUBLIC 
  */
 
+/**
+ * cryptui_keyset_new:
+ * @keytype: libcryptui key prefix
+ * @expand_keys: whether key's non-primary uids are part of the set
+ *
+ * Creates a new keylist
+ *
+ * Returns: the new keyset
+ */
 CryptUIKeyset*
-cryptui_keyset_new (const gchar *keytype)
+cryptui_keyset_new (const gchar *keytype, gboolean expand_keys)
 {
-    return g_object_new (CRYPTUI_TYPE_KEYSET, "keytype", keytype, NULL);
+    return g_object_new (CRYPTUI_TYPE_KEYSET, "keytype", keytype, 
+                         "expand-keys", expand_keys, NULL);
 }
 
+/**
+ * cryptui_keyset_has_key:
+ * @keyset: a libcryptui keyset
+ * @key: a libcryptui key
+ *
+ * Determines whether or not a key is present in the keyset
+ *
+ * Returns: TRUE if the key is present, FALSE otherwise
+ */
+gboolean
+cryptui_keyset_has_key (CryptUIKeyset *keyset, const gchar *key)
+{
+    return g_hash_table_lookup (keyset->priv->keys, key) ? TRUE : FALSE;
+}
+
+/**
+ * cryptui_keyset_get_keytype:
+ * @keyset: a libcryptui keyset
+ *
+ * Gets the libcryptui key prefix for the keyset
+ *
+ * Returns: a libcryptui key prefix
+ */
+const gchar*
+cryptui_keyset_get_keytype (CryptUIKeyset *keyset)
+{
+    return keyset->priv->keytype;
+}
+
+/**
+ * cryptui_keyset_get_keys:
+ * @keyset: a libcryptui keyset
+ *
+ * Gets a list of keys contained in the key set
+ *
+ * Returns: a doubly linked list of keys
+ */
 GList*             
 cryptui_keyset_get_keys (CryptUIKeyset *keyset)
 {
@@ -389,12 +435,26 @@ cryptui_keyset_get_keys (CryptUIKeyset *keyset)
     return keys;
 }
 
+/**
+ * cryptui_keyset_get_count:
+ * @keyset: a libcryptui keyset
+ *
+ * Gets the number of keys stored in the keyset
+ *
+ * Returns: the number of keys
+ */
 guint
 cryptui_keyset_get_count (CryptUIKeyset *keyset)
 {
     return g_hash_table_size (keyset->priv->keys);
 }
 
+/**
+ * cryptui_keyset_refresh:
+ * @keyset: a libcryptui keyset
+ *
+ * Checks the remote keyset to see which keys have been added or removed
+ */
 void
 cryptui_keyset_refresh (CryptUIKeyset *keyset)
 {
@@ -402,6 +462,8 @@ cryptui_keyset_refresh (CryptUIKeyset *keyset)
     gchar **k, **keys = NULL; 
     GError *error = NULL;
     
+    g_assert (keyset != NULL);
+    
     /* Make note of all the keys we had prior to refresh */
     check = g_hash_table_new (g_str_hash, g_str_equal);
     g_hash_table_foreach (keyset->priv->keys, (GHFunc)keys_to_hash, check);
@@ -432,6 +494,15 @@ finally:
     g_strfreev (keys);
 }
 
+/**
+ * cryptui_keyset_get_closure:
+ * @keyset: a libcryptui keyset
+ * @key: a libcryptui key
+ *
+ * TODO: Find out what closure is and document this function
+ *
+ * Returns: closure associated with key
+ */
 gpointer
 cryptui_keyset_get_closure (CryptUIKeyset *keyset, const gchar *key)
 {
@@ -445,6 +516,14 @@ cryptui_keyset_get_closure (CryptUIKeyset *keyset, const gchar *key)
     return closure;
 }
 
+/**
+ * cryptui_keyset_set_closure:
+ * @keyset: a libcryptui keyset
+ * @key: a libcryptui key
+ * @closure: TODO
+ *
+ * TODO: Find out what closure is and document this function
+ */
 void
 cryptui_keyset_set_closure (CryptUIKeyset *keyset, const gchar *key, 
                              gpointer closure)
@@ -459,6 +538,43 @@ cryptui_keyset_set_closure (CryptUIKeyset *keyset, const gchar *key,
     g_hash_table_insert (keyset->priv->keys, g_strdup (key), closure);    
 }
 
+/**
+ * cryptui_keyset_get_expand_keys:
+ * @keyset: a libcryptui keyset
+ *
+ * Gets whether or not non-primary key UIDs are included in the keyset
+ *
+ * Returns: TRUE if non-primary key UIDs are included in the keyset
+ */
+gboolean
+cryptui_keyset_get_expand_keys (CryptUIKeyset *keyset)
+{
+    gboolean expand;
+    g_object_get(keyset, "expand-keys", &expand, NULL);
+    return expand;
+}
+
+/**
+ * cryptui_keyset_set_expand_keys:
+ * @keyset: a libcryptui keyset
+ * @expand_keys: a gboolean
+ *
+ * Sets whether or not non-primary key UIDs are included in the keyset
+ */
+void
+cryptui_keyset_set_expand_keys (CryptUIKeyset *keyset, gboolean expand_keys)
+{
+    g_object_set(keyset, "expand-keys", expand_keys, NULL);
+}
+
+/**
+ * cryptui_keyset_cache_key:
+ * @keyset: a libcryptui keyset
+ * @key: libcryptui key to cache
+ *
+ * Stores the key's fields returned by the DBus method GetKeyFields in the
+ * keyset.
+ */
 void
 cryptui_keyset_cache_key (CryptUIKeyset *keyset, const gchar *key)
 {
@@ -470,7 +586,7 @@ cryptui_keyset_cache_key (CryptUIKeyset *keyset, const gchar *key)
     if (props)
         return;
 
-    if (!dbus_g_proxy_call (keyset->priv->remote_service, "GetKeyFields", &error,
+    if (!dbus_g_proxy_call (keyset->priv->remote_keyset, "GetKeyFields", &error,
                             G_TYPE_STRING, key, G_TYPE_STRV, cached_key_props, G_TYPE_INVALID, 
                             TYPE_G_STRING_VALUE_HASHTABLE, &props, G_TYPE_INVALID)) {
         g_warning ("dbus call to cache key failed: %s", error ? error->message : "");
@@ -484,6 +600,16 @@ cryptui_keyset_cache_key (CryptUIKeyset *keyset, const gchar *key)
         g_hash_table_remove (keyset->priv->key_props, key);
 }
 
+/**
+ * cryptui_keyset_key_get_string:
+ * @keyset: a libcryptui keyset
+ * @key: libcryptui key to fetch a property of
+ * @prop: string property to get
+ *
+ * Gets the given property of the key in the keyset.
+ *
+ * Returns: a string containing the property's value
+ */
 gchar*
 cryptui_keyset_key_get_string (CryptUIKeyset *keyset, const gchar *key, 
                                const gchar *prop)
@@ -507,6 +633,16 @@ cryptui_keyset_key_get_string (CryptUIKeyset *keyset, const gchar *key,
     return str;
 }
 
+/**
+ * cryptui_keyset_key_get_uint:
+ * @keyset: a libcryptui keyset
+ * @key: libcryptui key to fetch a property of
+ * @prop: uint property to get
+ *
+ * Gets the given property of the key in the keyset.
+ *
+ * Returns: a uint containing the property's value
+ */
 guint
 cryptui_keyset_key_get_uint (CryptUIKeyset *keyset, const gchar *key,
                              const gchar *prop)
@@ -531,20 +667,103 @@ cryptui_keyset_key_get_uint (CryptUIKeyset *keyset, const gchar *key,
 }
 
 
+/**
+ * cryptui_keyset_key_display_name:
+ * @keyset: a libcryptui keyset
+ * @key: a libcryptui key
+ *
+ * Gets the "display-name" property of the given key
+ *
+ * Returns: the display name of the key
+ */
 gchar*
 cryptui_keyset_key_display_name (CryptUIKeyset *keyset, const gchar *key)
 {
     return cryptui_keyset_key_get_string (keyset, key, "display-name");
 }
 
+/**
+ * cryptui_keyset_key_display_id:
+ * @keyset: a libcryptui keyset
+ * @key: a libcryptui key
+ *
+ * Gets the "display-id" property of the given key
+ *
+ * Returns: the display id of the key
+ */
 gchar*
 cryptui_keyset_key_display_id (CryptUIKeyset *keyset, const gchar *key)
 {
     return cryptui_keyset_key_get_string (keyset, key, "display-id");
 }
 
+/**
+ * cryptui_keyset_key_flags:
+ * @keyset: a libcryptui keyset
+ * @key: a libcryptui key
+ *
+ * Gets the key's flags
+ *
+ * Returns: a uint containing the flags
+ */
 guint
 cryptui_keyset_key_flags (CryptUIKeyset *keyset, const gchar *key)
 {
     return cryptui_keyset_key_get_uint (keyset, key, "flags");
 }
+
+/**
+ * cryptui_keyset_key_raw_keyid:
+ * @keyset: a libcryptui keyset
+ * @key: a libcryptui key
+ *
+ * Gets the key's raw key id
+ *
+ * Returns: a string with the raw key id
+ */
+gchar*
+cryptui_keyset_key_raw_keyid (CryptUIKeyset *keyset, const gchar *key)
+{
+    return cryptui_keyset_key_get_string (keyset, key, "raw-id");
+}
+
+/**
+ * cryptui_keyset_keys_raw_keyids:
+ * @keyset: a libcryptui keyset
+ * @keys: an array of libcryptui keys
+ *
+ * Gets the keys' raw key ids
+ *
+ * Returns: an array of raw key ids
+ */
+gchar**
+cryptui_keyset_keys_raw_keyids (CryptUIKeyset *keyset, const gchar **keys)
+{
+    guint nkeys;
+    const gchar **k;
+    gchar **ids;
+    int i;
+    
+    /* TODO: This could get inefficient */
+    
+    for (k = keys, nkeys = 0; *k; k++)
+        nkeys++;
+    ids = g_new0 (gchar*, nkeys + 1);
+    for (k = keys, i = 0; *k; k++, i++)
+        ids[i] = cryptui_keyset_key_get_string (keyset, *k, "raw-id");
+    
+    return ids;
+}
+
+/* -----------------------------------------------------------------------------
+ * INTERNAL
+ */
+
+const gchar*
+_cryptui_keyset_get_internal_keyid (CryptUIKeyset *keyset, const gchar *keyid)
+{
+    gpointer orig_key, value;
+    if (!g_hash_table_lookup_extended (keyset->priv->keys, keyid, &orig_key, &value))
+        return NULL;
+    return (const gchar*)orig_key;
+}
